@@ -134,6 +134,8 @@ enum
 #define MAX_DIGITS     200         /* Maximum displayable number of digits. */
 #define MAX_LOCALIZED  (MAX_DIGITS * (1 + MB_LEN_MAX) + MB_LEN_MAX)
 
+#define DEFAULT_ACCURACY 9
+
 #ifndef MAXLINE
 #define MAXLINE        512        /* Length of character strings. */
 #endif /*MAXLINE*/
@@ -177,11 +179,11 @@ struct button {
     enum button_flags flags; /* Misc flags */
 };
 
-struct exprm_state {       /* Expression mode state */
-    struct button button;  /* Current button/character pressed. */
-    int value;
+/* Expression mode state */
+struct exprm_state {
     int ans[MP_SIZE];      /* Previously calculated answer */
     char *expression;      /* Expression entered by user */
+    int cursor;
 };
 
 /* Circular list of Arithmetic Precedence Mode states*/ 
@@ -234,8 +236,6 @@ struct calcVars {                      /* Calctool variables and options. */
     int accuracy;      /* Number of digits precision (Max 9). */
     int cur_op;        /* Current arithmetic operation. */
     int error;         /* Indicates some kind of display error. */
-    int ghost_zero;    /* Flag to indicate display with "0", actually 
-                           having empty content. */
     int math_error;    /* Math error (used in expression mode) */
     int key_exp;       /* Set if entering exponent number. */
     int new_input;     /* New number input since last op. */
@@ -247,7 +247,6 @@ struct calcVars {                      /* Calctool variables and options. */
     int show_tsep;     /* Set if the thousands seperator should be shown. */
     int show_zeroes;   /* Set if trailing zeroes should be shown. */
     int toclear;       /* Indicates if display should be cleared. */
-    int bitcalculating_mode;  /* 0 = no, else yes */
 };
 
 typedef struct calcVars *Vars;
